package nckd.jxccl.hr.hstu.plugin.form;

import kd.bos.context.RequestContext;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.form.CloseCallBack;
import kd.bos.form.ConfirmCallBackListener;
import kd.bos.form.MessageBoxOptions;
import kd.bos.form.MessageBoxResult;
import kd.bos.form.control.Control;
import kd.bos.form.events.ClosedCallBackEvent;
import kd.bos.form.events.MessageBoxClosedEvent;
import kd.bos.form.plugin.AbstractFormPlugin;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.schedule.api.JobInfo;
import kd.bos.schedule.api.JobType;
import kd.bos.schedule.form.JobForm;
import kd.bos.schedule.form.JobFormInfo;
import kd.bos.servicehelper.QueryServiceHelper;
import kd.sdk.plugin.Plugin;
import nckd.jxccl.base.common.constant.FormConstant;
import nckd.jxccl.base.common.utils.ConvertUtil;
import nckd.jxccl.base.common.utils.StrFormatter;
import nckd.jxccl.hr.hstu.business.CreateEvalQuestService;
import nckd.jxccl.hr.hstu.common.HonorStudentConstant;
import nckd.jxccl.hr.hstu.task.CreateEvalQuestTask;
import nckd.jxccl.hr.hstu.task.CreateEvalQuestTaskClick;

import java.util.ArrayList;
import java.util.EventObject;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.StringJoiner;

/**
* 生成评价对象表单插件
* @author W.Y.C
* @date 2025/7/1 10:19
* @version 1.0
*/
public class CreateEvalQuestFormPlugin extends AbstractFormPlugin implements Plugin {

    @Override
    public void registerListener(EventObject e) {
        this.addClickListeners(HonorStudentConstant.OP_BTN_OK);
    }

    @Override
    public void afterCreateNewData(EventObject e) {
        super.afterCreateNewData(e);
        Long evalQuestTplId = ConvertUtil.toLong(this.getView().getFormShowParameter().getCustomParams().get(HonorStudentConstant.NCKD_EVALQUESTTPLID));
        this.getModel().setValue(HonorStudentConstant.NCKD_EVALQUESTTPLID, evalQuestTplId);
    }

    @Override
    public void click(EventObject evt) {
        Control source = (Control) evt.getSource();
        if (HonorStudentConstant.OP_BTN_OK.equals(source.getKey())) {
            DynamicObject selectedEvalRule = (DynamicObject)this.getModel().getValue(HonorStudentConstant.NCKD_SELECTEDEVALRULE);
            if(selectedEvalRule == null){
                this.getView().showTipNotification("请选择评价规则");
                return;
            }
           DynamicObjectCollection selectedPerson = (DynamicObjectCollection)this.getModel().getValue(HonorStudentConstant.NCKD_SELECTEDPERSON);
            if(selectedPerson.isEmpty()){
                this.getView().showTipNotification("请选择评价对象");
                return;
            }

            Map<String, Object> returnMap = new HashMap<>(2);
            returnMap.put("selectedEvalRule", selectedEvalRule);
            returnMap.put("selectedPerson", selectedPerson);
            this.getView().showConfirm("是否确认生成已选人员的评测问卷？", MessageBoxOptions.YesNo, new ConfirmCallBackListener(HonorStudentConstant.SUBMIT_KEY, this));
        }
    }

    @Override
    public void confirmCallBack(MessageBoxClosedEvent messageBoxClosedEvent) {
        String callBackId = messageBoxClosedEvent.getCallBackId();
        if (HonorStudentConstant.SUBMIT_KEY.equals(callBackId) && messageBoxClosedEvent.getResult() == MessageBoxResult.Yes) {
            //批量新增综合测评问卷对话框确认事件
            Long evalQuestTplId = ConvertUtil.toLong(this.getView().getFormShowParameter().getCustomParams().get(HonorStudentConstant.NCKD_EVALQUESTTPLID));
            DynamicObject selectedEvalRule = (DynamicObject) this.getModel().getValue(HonorStudentConstant.NCKD_SELECTEDEVALRULE);
            DynamicObjectCollection selectedPerson = (DynamicObjectCollection) this.getModel().getValue(HonorStudentConstant.NCKD_SELECTEDPERSON);
            //------------ 校验是否已生成过问卷（校验规则：评价对象是否已经有对应的评价规则或模板问卷） begin ------------
            List<Long> idList = new ArrayList<>();
            selectedPerson.forEach(map ->{
                DynamicObject depEmpRow = (DynamicObject)map.getDynamicObject(FormConstant.BASEDATAID_KEY);
                idList.add(depEmpRow.getLong(HonorStudentConstant.ID_KEY));
            });
            QFilter filter = new QFilter(HonorStudentConstant.NCKD_EVALPERSON, QCP.in, idList);
            filter.and(
                    new QFilter(HonorStudentConstant.NCKD_EVALUATIONRULE,QCP.equals,selectedEvalRule.getLong(HonorStudentConstant.ID_KEY))
                            .or(new QFilter(HonorStudentConstant.NCKD_EVALQUESTTPL,QCP.equals,evalQuestTplId))
            );
            DynamicObjectCollection evalQuestColl = QueryServiceHelper.query(HonorStudentConstant.NCKD_EVALQUEST,HonorStudentConstant.NCKD_EVALPERSON+".person.name",new QFilter[]{filter});
            if(!evalQuestColl.isEmpty()){
                StringJoiner exitsPersonName = new StringJoiner("，");
                evalQuestColl.forEach(dynamicObject -> exitsPersonName.add(dynamicObject.getString(HonorStudentConstant.NCKD_EVALPERSON+".person.name")));
                this.getView().showErrorNotification(StrFormatter.format("下列人员已存在问卷：{}；不能重复生成，请检查！",exitsPersonName.toString()));
                return;
            }
            //------------ 校验是否已生成过问卷（校验规则：评价对象是否已经有对应的评价规则或模板问卷） end ------------

            if (selectedPerson.size() > 10) {
                //选择生成人员超过10人开启异步任务
                // 1. 创建任务实例
                JobInfo jobInfo = new JobInfo();
                jobInfo.setTaskClassname(CreateEvalQuestTask.class.getName());
                Map<String, Object> params = new HashMap<>();
                params.put(HonorStudentConstant.NCKD_EVALQUESTTPLID, evalQuestTplId);
                params.put(HonorStudentConstant.NCKD_SELECTEDEVALRULE, selectedEvalRule);
                params.put(HonorStudentConstant.NCKD_SELECTEDPERSON, selectedPerson);
                //调度任务中获取不到当前用户信息
                long currUserId = RequestContext.get().getCurrUserId();
                long orgId = RequestContext.get().getOrgId();
                params.put(HonorStudentConstant.CURR_USER_ID, currUserId);
                params.put(HonorStudentConstant.CURR_ORG_ID, orgId);
                jobInfo.setParams(params);
                jobInfo.setJobType(JobType.REALTIME);
                jobInfo.setSuccessNotify(Boolean.TRUE);

                // 2. 配置进度界面
                JobFormInfo formInfo = new JobFormInfo(jobInfo);
                formInfo.setCaption("生成综合评测问卷进度");
                formInfo.setCloseCallBack(new CloseCallBack(EvalQuestTplListPlugin.class.getName(), HonorStudentConstant.TASK_CREATEEVAL));
                formInfo.setClickClassName(CreateEvalQuestTaskClick.class.getName());
                // 允许用户中途取消
                //formInfo.setCanStop(true);
                // 允许切到后台
                //formInfo.setCanBackground(true);

                // 3. 发布任务并显示进度界面
                JobForm.dispatch(formInfo, this.getView().getParentView());

                this.getView().close();
            } else {
                try {
                    CreateEvalQuestService.batchGenerateEvalQuest(evalQuestTplId, selectedEvalRule, selectedPerson, null, null, null);
                    this.getView().getParentView().showSuccessNotification("生成综合评测问卷成功，可在【查看评价列表】中查看");
                    this.getView().close();
                } catch (Exception e) {
                    this.getView().showErrorNotification("生成评价问卷时出错，原因： " + e.getMessage());
                }
            }

        }
    }

    @Override
    public void closedCallBack(ClosedCallBackEvent closedCallBackEvent) {
        System.out.println();
    }
}